#include <iostream>
#include <vector>
#include <string>
#include "svg.h"
using namespace std;

const auto IMAGE_WIDTH = 400;
const auto IMAGE_HEIGHT = 300;
const auto TEXT_LEFT = 20;
const auto TEXT_BASELINE = 20;
const auto TEXT_WIDTH = 50;
const auto BIN_HEIGHT = 30;
const auto BLOCK_WIDTH = 10;
const auto MAX_WIDTH = IMAGE_WIDTH - TEXT_WIDTH;

void
svg_text(double left, double baseline, int font_size, string text) {
    cout << "<text x='" << left << "' y='" << baseline << "' font-size='" << font_size << "'>" << text << "</text>";
}

void
svg_rect(double x, double y, double width, double height, string stroke = "black", string fill = "aaaaaa") {
    cout << "<rect x='" << x<< "' y='" << y<< "' width='" << width<< "' height='" << height<< "' stroke='" << stroke<< "' fill='" << fill << "' />\n";
}

void
svg_begin(double width, double height) {
    cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    cout << "<svg ";
    cout << "width='" << width << "' ";
    cout << "height='" << height << "' ";
    cout << "viewBox='0 0 " << width << " " << height << "' ";
    cout << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void
svg_end() {
    cout << "</svg>\n";
}

void
proverka(bool& flag, int font_size){
        if (font_size < 8) {
            cerr << "Font size is too small. Please enter a value between 8 and 32." << endl;
            flag = true;
        } else if (font_size > 32) {
            cerr << "Font size is too large. Please enter a value between 8 and 32." << endl;
            flag = true;
        } else {
            flag = false;
        }
}

void
show_histogram_svg(const vector<size_t>& bins) {
    int font_size = 12;
    bool flag = true;
    while(flag) {
        cerr << "Enter font size (8-32): ";
        cin >> font_size;
        proverka(flag, font_size);
    }
    const auto MAX_WIDTH = IMAGE_WIDTH - TEXT_WIDTH;
    size_t max_count = 0;
    for (size_t x : bins) {
        if (x > max_count) {
            max_count = x;
        }
    }
    if (max_count == 0) {
        max_count = 1;
    }
    auto scale_factor = static_cast<double>(MAX_WIDTH) / (max_count * BLOCK_WIDTH);
    if (scale_factor > 1) {
        scale_factor = 1;
    }
    svg_begin(400, 300);
    double top = 0;
    for (size_t bin : bins) {
        double bin_width = BLOCK_WIDTH * bin * scale_factor;
        svg_text(TEXT_LEFT, top + TEXT_BASELINE, font_size, to_string(bin));
        svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
        top += BIN_HEIGHT;
    }
    svg_end();
}
